<?php

namespace Drupal\refields\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeRepositoryInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\refields\EntityHelpers;
use Symfony\Component\DependencyInjection\ContainerInterface;

final class UsageController extends ControllerBase {

  use EntityHelpers, StringTranslationTrait;

  public function __construct(
    private readonly EntityFieldManagerInterface $entityFieldManager,
    private readonly EntityTypeBundleInfoInterface $entityTypeBundleInfo,
    private readonly EntityTypeRepositoryInterface $entityTypeRepository,
  ) {}

  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get(EntityFieldManagerInterface::class),
      $container->get(EntityTypeBundleInfoInterface::class),
      $container->get(EntityTypeRepositoryInterface::class),
    );
  }

  public function __invoke(): array {
    $build = [];

    $entity_types = $this->listFieldableEntityTypes();
    foreach ($entity_types as $entity_type_id => $label) {
      $definitions = $this->listEnabledFieldStorageDefinitionsForEntityType($entity_type_id);
      if (empty($definitions)) {
        continue;
      }

      $build[$entity_type_id] = [
        '#type' => 'details',
        '#title' => $label,
        '#open' => TRUE,
      ];

      $bundles = $this->listBundles($entity_type_id);
      $build[$entity_type_id]['overview'] = [
        '#type' => 'table',
        '#header' => array_merge([$this->t('Field')], $bundles),
      ];

      foreach ($definitions as $definition) {
        $build[$entity_type_id]['overview']['#rows'][$definition->label()] = array_merge(
          [$definition->getName()],
          array_map(function(string $bundle) use ($definition, $entity_type_id): string {
            $excluded_bundles = $definition->getThirdPartySetting(
              'refields',
              'refields_excluded_bundles',
              []
            );

            if (!in_array($bundle, $excluded_bundles)) {
              return 'Included';
            }

            if ($this->fieldExists($definition->getName(), $entity_type_id, $bundle)) {
              return 'Excluded (field exists)';
            }

            return 'Excluded';
          }, array_keys($bundles))
        );
      }
    }

    return $build;
  }

  /**
   * List all enabled field storage definitions for a given entity type.
   *
   * @param string $entity_type_id
   *
   * @return \Drupal\field\FieldStorageConfigInterface[]
   */
  private function listEnabledFieldStorageDefinitionsForEntityType(string $entity_type_id): array {
    try {
      return array_filter(
        $this->entityFieldManager->getFieldStorageDefinitions($entity_type_id),
        fn(FieldStorageDefinitionInterface $definition): bool => $definition instanceof FieldStorageConfigInterface
          && $definition->getThirdPartySetting('refields', 'refields_enabled', FALSE)
      );
    }
    catch (\Exception) {
      return [];
    }
  }

  /**
   * @return \Drupal\Core\Entity\FieldableEntityInterface[]
   */
  private function listFieldableEntityTypes(): array {
    return $this->entityTypeRepository->getEntityTypeLabels();
  }

}
