<?php

namespace Drupal\refields\Fields;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\field_ui\Form\FieldStorageCreationTrait;
use Drupal\refields\EntityHelpers;

abstract class BaseFieldCreator {

  use EntityHelpers, FieldStorageCreationTrait;

  public function __construct(
    protected EntityFieldManagerInterface $entityFieldManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {}

  protected function performFieldSetup(
    string $bundle,
    FieldStorageDefinitionInterface $definition
  ): void {
    $field_name = $definition->getName();
    $entity_type_id = $definition->getTargetEntityTypeId();

    $default_options = $this->getExistingDefaults($field_name, $entity_type_id);

    $this->entityTypeManager->getStorage('field_config')->create([
      ...$default_options['field_config'] ?? [],
      'field_name' => $field_name,
      'entity_type' => $entity_type_id,
      'bundle' => $bundle,
      'translatable' => $definition->isTranslatable(),
    ])->save();

    try {
      @$this->configureEntityFormDisplay($field_name, $default_options['entity_form_display'] ?? []);
    }
    catch (\Exception) {
      // Ignore.
    }
    try {
      @$this->configureEntityViewDisplay($field_name, $default_options['entity_view_display'] ?? []);
    }
    catch (\Exception) {
      // Ignore.
    }
  }

  /**
   * Method borrowed, see
   * \Drupal\field_ui\Form\FieldStorageReuseForm::getExistingFieldDefaults().
   *
   * @param string $field_name
   * @param string $entity_type_id
   *
   * @return array
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getExistingDefaults(string $field_name, string $entity_type_id): array {
    $default_options = [];
    $field_map = $this->entityFieldManager->getFieldMap();
    if (empty($field_map[$entity_type_id][$field_name]['bundles'])) {
      return [];
    }

    $bundles = $field_map[$entity_type_id][$field_name]['bundles'];
    sort($bundles);
    $existing_bundle = reset($bundles);

    $definitions = $this->entityFieldManager->getFieldDefinitions(
      $entity_type_id,
      $existing_bundle
    );

    $default_options['field_config'] = [
      'description' => $definitions[$field_name]->getDescription(),
      'label' => $definitions[$field_name]->getLabel(),
      'settings' => $definitions[$field_name]->getSettings(),
      'required' => $definitions[$field_name]->isRequired(),
      'default_value' => $definitions[$field_name]->getDefaultValueLiteral(),
      'default_value_callback' => $definitions[$field_name]->getDefaultValueCallback(),
    ];

    foreach (['entity_form_display', 'entity_view_display'] as $display_type) {
      $properties = [
        'targetEntityType' => $entity_type_id,
        'bundle' => $existing_bundle,
      ];

      /** @var \Drupal\Core\Entity\Display\EntityDisplayInterface[] $displays */
      $displays = $this->entityTypeManager
        ->getStorage($display_type)
        ->loadByProperties($properties);

      foreach ($displays as $display) {
        if ($settings = $display->getComponent($field_name)) {
          $default_options[$display_type][$display->getMode()] = array_merge(
            $settings,
            $properties
          );
        }
      }
    }

    return $default_options;
  }

}
