<?php

namespace Drupal\refields\Form;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\refields\EntityHelpers;
use Drupal\refields\Fields\CreateFieldsOnFieldConfigUpdate;

/**
 * Add the settings to the field configuration form.
 */
final class AddRefieldsSettingsToFieldConfig {

  use EntityHelpers, StringTranslationTrait, DependencySerializationTrait;

  public function __construct(
    private CreateFieldsOnFieldConfigUpdate $createFieldsOnFieldConfigUpdate,
  ) {}

  /**
   * Add the settings to the field configuration form.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return void
   */
  public function addSettings(
    array &$form,
    FormStateInterface $form_state
  ): void {
    /** @var \Drupal\Core\Field\FieldConfigInterface $field_definition */
    $field_definition = $form_state->getFormObject()?->getEntity();
    $field_storage_definition = $field_definition?->getFieldStorageDefinition();
    if (!$field_storage_definition instanceof FieldStorageConfigInterface) {
      return;
    }

    $form['refields'] = [
      '#type' => 'details',
      '#title' => $this->t('Re-Fields settings'),
      '#group' => 'additional_settings',
      '#tree' => TRUE,
    ];

    $form['refields']['refields_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Re-Fields'),
      '#description' => $this->t('If enabled, Re-Fields will automatically add this field to existing and new future bundles.'),
      '#default_value' => $field_storage_definition->getThirdPartySetting('refields', 'refields_enabled', FALSE),
    ];

    $form['refields']['refields_excluded_bundles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Excluded bundles'),
      '#description' => $this->t('The selected bundles will be ignored when automatically creating the fields.'),
      '#options' => $this->listBundles($field_storage_definition->getTargetEntityTypeId()),
      '#default_value' => $field_storage_definition->getThirdPartySetting('refields', 'refields_excluded_bundles', []),
    ];

    $form['actions']['submit']['#submit'][] = [$this, 'storeSettings'];
    $form['actions']['submit']['#submit'][] = [$this, 'triggerFieldsUpdate'];
  }

  /**
   * Store the settings for the field.
   *
   * @param array $form
   * @param FormStateInterface $form_state
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function storeSettings(
    array &$form,
    FormStateInterface $form_state
  ): void {
    /** @var \Drupal\Core\Field\FieldConfigInterface $field_definition */
    $field_definition = $form_state->getFormObject()?->getEntity();
    $field_storage_definition = $field_definition?->getFieldStorageDefinition();
    if (!$field_storage_definition instanceof FieldStorageConfigInterface) {
      return;
    }

    // When creating a new field, it should exist by now. Making sure the field
    // configuration is up-to-date.
    if ($field_storage_definition->isNew()) {
      $field_storage_definition = FieldStorageConfig::load($field_storage_definition->id());
    }

    $field_storage_definition->setThirdPartySetting(
      'refields',
      'refields_enabled',
      $form_state->getValue(['refields', 'refields_enabled'])
    );

    $field_storage_definition->setThirdPartySetting(
      'refields',
      'refields_excluded_bundles',
      array_filter($form_state->getValue(['refields', 'refields_excluded_bundles']))
    );

    $field_storage_definition->save();
  }

  public function triggerFieldsUpdate(
    array &$form,
    FormStateInterface $form_state
  ): void {
    /** @var \Drupal\Core\Field\FieldConfigInterface $field_definition */
    $field_definition = $form_state->getFormObject()?->getEntity();
    $field_storage_definition = $field_definition?->getFieldStorageDefinition();
    if (!$field_storage_definition instanceof FieldStorageConfigInterface) {
      return;
    }

    $this->createFieldsOnFieldConfigUpdate->createFields($field_storage_definition);
  }

}
